@@ -1,6 +1,6 @@
 /* module.c - the module itself
  *
- * Copyright (C) 2004-2007 Gerhard Hring <gh@ghaering.de>
+ * Copyright (C) 2004-2010 Gerhard Hring <gh@ghaering.de>
  *
  * This file is part of pysqlite.
  *
@@ -29,6 +29,10 @@
 #include "microprotocols.h"
 #include "row.h"
 
+#ifdef PYSQLITE_EXPERIMENTAL
+#include "backup.h"
+#endif
+
 #if SQLITE_VERSION_NUMBER >= 3003003
 #define HAVE_SHARED_CACHE
 #endif
@@ -314,6 +318,9 @@ PyMODINIT_FUNC init_sqlite(void)
         (pysqlite_connection_setup_types() < 0) ||
         (pysqlite_cache_setup_types() < 0) ||
         (pysqlite_statement_setup_types() < 0) ||
+        #ifdef PYSQLITE_EXPERIMENTAL
+        (pysqlite_backup_setup_types() < 0) ||
+        #endif
         (pysqlite_prepare_protocol_setup_types() < 0)
        ) {
         return;
@@ -445,7 +452,9 @@ PyMODINIT_FUNC init_sqlite(void)
      * threads have already been initialized.
      *  (see pybsddb-users mailing list post on 2002-08-07)
      */
+#ifdef WITH_THREAD
     PyEval_InitThreads();
+#endif
 
 error:
     if (PyErr_Occurred())
